#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# get_account_balance_RBS                                                      #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program returns the balance and recent transactions of an RBS account   #
# using Firefox and Selenium. The default behaviour is to open the RBS web     #
# interface and to display the balance and recent transactions in the          #
# terminal, leaving the browser open. The behaviour instead can be set to loop #
# with a certain time interval, closing the browser and displaying the balance #
# and recent transactions in the terminal continuously.                        #
#                                                                              #
# copyright (C) 2017 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help              display help message
    --version               display version and exit

    --loop
    --interval=INT          time between recordings (s)    [default: 120]

    --credentials=FILEPATH  credentials file               [default: ~/.rbs]

    --sleep_time=INT        time between web interactions  [default: 1]
"""

import docopt
import time

import denarius.RBS

name    = "get_account_balance_RBS"
version = "2018-01-21T0227Z"
logo    = None

def main(options):

    loop                 =     options["--loop"]
    interval             = int(options["--interval"])
    filepath_credentials =     options["--credentials"]
    sleep_time           = int(options["--sleep_time"])

    if loop:
        while True:
            text = denarius.RBS.printout(
                filepath_credentials = filepath_credentials,
                log_out              = True,
                close_driver         = True,
                sleep_time           = sleep_time
            )
            print(text)
            time.sleep(interval)
    else:
        text = denarius.RBS.printout(
            filepath_credentials = filepath_credentials,
            log_out              = False,
            close_driver         = False,
            sleep_time           = sleep_time
        )
        print(text)

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/env python

"""
################################################################################
#                                                                              #
# print_Kraken_last_price_BTC_EUR                                              #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program prints the last Kraken Bitcoin price in Euros.                  #
#                                                                              #
# copyright (C) 2018 William Breaden Madden                                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################
"""

import sys

import denarius.Kraken

name    = "print_Kraken_last_price_BTC_EUR"
version = "2018-01-31T0002Z"

def main():

    denarius.Kraken.start_API()
    print(denarius.Kraken.last_price_XBT_EUR())

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# print_table_bank_account                                                     #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program prints a table of the transactions of a bank account.           #
#                                                                              #
# copyright (C) 2018 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help   display help message
    --version    display version and exit

    --bank=TEXT  bank to access  [default: Monzo]
"""

import docopt

import denarius.banks

name    = "print_table_bank_account"
version = "2018-02-06T1956Z"

def main(options):

    denarius.banks.transactions_DataFrame_bank(bank = options["--bank"], print_table = True)

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# Santander                                                                    #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program provides banking and other utilities.                           #
#                                                                              #
# copyright (C) 2018 William Breaden Madden                                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################
"""

import decimal
import docopt
import os
import re
import sys
import time

import numpy as np
import pandas as pd
from selenium import webdriver
from selenium.webdriver.common.keys import Keys

name    = "Santander"
version = "2018-01-18T1452Z"

def transactions_DataFrame(
    filepath_credentials               = "~/.santander",
    convert_currency_strings_to_floats = True,
    log_out                            = True,
    close_driver                       = True,
    sleep_time                         = 6
    ):

    """
    Access Santander online banking using credentials from a file and return
    recent transactions as a DataFrame.
    """

    # access credentials from file
    filepath_credentials = os.path.expanduser(filepath_credentials)
    if not os.path.isfile(filepath_credentials):
        print("no credentials file {filepath} found".format(filepath = filepath_credentials))
        sys.exit()
    with open(filepath_credentials, "r") as file_credentials:
        credentials_string = file_credentials.read()
        credentials = {}
        exec(credentials_string, credentials)
        customer_number          = credentials["customer_number"]
        customer_PIN             = credentials["customer_PIN"]
        security_question_answer = credentials["security_question_answer"]

    # access transactions via banking website
    try:
        driver = webdriver.Firefox()
        driver.get("https://retail.santander.co.uk/LOGSUK_NS_ENS/BtoChannelDriver.ssobto?dse_operationName=LOGON")
        time.sleep(sleep_time)
        # enter customer number
        try:
            element = driver.find_element_by_name("infoLDAP_E.customerID")
            element.send_keys(customer_number)
            # login button
            button_login = driver.find_element_by_id("btnFO")
            button_login.click()
            time.sleep(sleep_time)
        except:
            pass
        ## security question
        try:
            element = driver.find_element_by_name("cbQuestionChallenge.responseUser")
            element.send_keys(security_question_answer)
            # continue button
            button_login = driver.find_element_by_name("buttons.1")
            button_login.click()
            time.sleep(sleep_time)
        except:
            pass
        # enter customer PIN
        try:
            element = driver.find_element_by_name("authentication.CustomerPIN")
            element.send_keys(customer_PIN)
            # continue button
            button_login = driver.find_element_by_name("buttons.1")
            button_login.click()
            time.sleep(sleep_time)
        except:
            pass
        # view transactions
        try:
            driver.get("https://retail.santander.co.uk/EBAN_Accounts_ENS/BtoChannelDriver.ssobto?dse_operationName=CheckForErrors&opCode=00000002&url=%2FEBAN_Accounts_ENS%2FBtoChannelDriver.ssobto%3Fdse_operationName%3DViewTransactions")
            time.sleep(sleep_time)
            # get transactions table
            transactions = driver.find_element_by_class_name("cardlytics_history_table.data")
            # convert transactions table to DataFrame
            df = pd.DataFrame(columns = ["date", "description", "money_in", "money_out", "balance"])
            for row in transactions.find_elements_by_tag_name("tr"):
                columns = row.find_elements_by_tag_name("td")
                columns = [column.text for column in columns]
                if columns:
                    df = df.append(
                        {
                            "date":        columns[0],
                            "description": columns[1],
                            "money_in":    columns[2],
                            "money_out":   columns[3],
                            "balance":     columns[4]
                        },
                        ignore_index = True
                    )
        except:
            pass
        if log_out:
            driver.get("https://retail.santander.co.uk/EBAN_Accounts_ENS/channel.ssobto?dse_operationName=contentTemplate&iurl=%2FLOGSUK_S_ENS%2Fchannel.ssobto%3Fdse_operationName%3DLOGOFF&lang=en-GB&opCode=00000098&layout=transaction")
        if close_driver:
            driver.quit()
        if convert_currency_strings_to_floats:
            df["money_in"]  = df.apply(lambda x: currency_string_to_float(x["money_in"]),  axis = 1)
            df["money_out"] = df.apply(lambda x: currency_string_to_float(x["money_out"]), axis = 1)
            df["balance"]   = df.apply(lambda x: currency_string_to_float(x["balance"]),   axis = 1)
    except:
        print("error accessing transactions")
        try:
            driver.quit()
        except:
            pass
        return None
    return df

def loop_save_transactions_to_CSV(
    filepath_CSV = "Santander.csv",
    sleep_time   = 6,
    interval     = 30
    ):

    """
    Get current transactions online. Open previously-recorded transactions, if
    any, and merge them with the current transactions to avoid duplications.
    Save the merged transactions to CSV. Do this in a loop.
    """

    while True:
        try:
            # get transactions
            df = transactions_DataFrame(sleep_time = sleep_time)
            # check existing recorded data to avoid saving previously-saved data
            if os.path.isfile(filepath_CSV):
                try:
                    df_old = pd.read_csv(filepath_CSV)
                    df = pd.concat([df_old, df], ignore_index = True).drop_duplicates().reset_index(drop = True)
                    header = False
                except:
                    print("error reading from file {filepath_CSV}".format(filepath_CSV = filepath_CSV))
                    header = True
            print(df)
            # save transactions to CSV
            with open(filepath_CSV, "a") as file_CSV:
                print("save transactions to CSV {filepath}".format(filepath = filepath_CSV))
                df.to_csv(file_CSV, header = header, index = False)
        except:
            print("error accessing and recording transactions")
            pass
        print("sleep {interval} s".format(interval = interval))
        time.sleep(interval)

def payment_in_transactions_CSV(
    reference    = None,
    value        = None,
    filepath_CSV = "Santander.csv"
    ):

    """
    Access transactions recorded in CSV and load them as a DataFrame. Search
    the "description" field of the DataFrame for a specified reference. If the
    reference is found, check the value of the payment is correct by comparing
    the specified value with the "money_in" field.
    """

    if not reference or not value:
        return False
    if not os.path.isfile(filepath_CSV):
        print("file {filepath} not found".format(filepath = filepath_CSV))
        sys.exit()
    df = pd.read_csv(filepath_CSV)
    match = df[df["description"].str.contains(reference)]
    if not match.empty:
        return value == match["money_in"].values[0]
    else:
        return False

def currency_string_to_float(amount):

    """
    Convert a string of the form "£2,718.28" or "-£27.18" to a float with
    appropriate sign.
    """

    try:
        value = float(decimal.Decimal(re.sub(r"[^\d.]", "", amount)))
        if amount[0] == "-":
            value = -value
        return value
    except:
        pass
    return float(0)



# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# arbitrage                                                                    #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program provides arbitrage utilities.                                   #
#                                                                              #
# copyright (C) 2018 William Breaden Madden                                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################
"""

from __future__ import division
import datetime
import json
import logging
import os
import random
import requests
import time

from bs4 import BeautifulSoup
import currency_converter
import pandas as pd
import pyprel
import technicolor

__name__    = "arbitrage"
__version__ = "2018-03-13T2151Z"

log = logging.getLogger(__name__)
log.addHandler(technicolor.ColorisingStreamHandler())
log.setLevel(logging.INFO)

def DataFrame_LBC_UK_buy_ads():
    """
    Return a DataFrame of current LocalBitcoins UK buy ads.
    """
    log.info("get LocalBitcoins UK buy ads DataFrame")
    timestamp = datetime.datetime.utcnow()
    LBC_UK_buy_ads = [ad for ad in requests.get("https://localbitcoins.com/buy-bitcoins-online/GB/united-kingdom/national-bank-transfer/.json").json()["data"]["ad_list"]]
    df = pd.DataFrame(columns = ["datetime", "temp_price_gbp", "seller_index", "seller_username", "seller_feedback_score", "seller_trade_count"])
    for index, ad in enumerate(LBC_UK_buy_ads):
        df = df.append(
            {
                "datetime"                     : timestamp,
                "seller_index"                 : index,
                "temp_price_gbp"               : float(ad["data"]["temp_price"]),
                "seller_username"              :       ad["data"]["profile"]["username"],
                "seller_feedback_score"        : float(ad["data"]["profile"]["feedback_score"]),
                "seller_trade_count"           :       ad["data"]["profile"]["trade_count"],
                "max_amount"                   :       ad["data"]["max_amount"],
                "min_amount"                   :       ad["data"]["min_amount"],
                "ad_id"                        :       ad["data"]["ad_id"],
                "bank_name"                    :       ad["data"]["bank_name"],
                "online_provider"              :       ad["data"]["online_provider"],
                "payment_window_minutes"       : float(ad["data"]["payment_window_minutes"]),
                "require_feedback_score"       : float(ad["data"]["require_feedback_score"]),
                "require_identification"       :       ad["data"]["require_identification"],
                "require_trade_volume"         : float(ad["data"]["require_trade_volume"]),
                "require_trusted_by_advertiser":       ad["data"]["require_trusted_by_advertiser"],
                "sms_verification_required"    :       ad["data"]["sms_verification_required"],
                "trade_type"                   :       ad["data"]["trade_type"],
                "trusted_required"             :       ad["data"]["trusted_required"],
                "volume_coefficient_btc"       : float(ad["data"]["volume_coefficient_btc"])
            },
            ignore_index = True
        )
    return df

def DataFrame_Kraken_ticker_BTC_EUR():
    """
    Return a DataFrame of current and recent Kraken Bitcoin prices expressed in Euros.
    """
    log.info("get Kraken ticker DataFrame")
    Kraken_ticker = requests.get("https://api.kraken.com/0/public/Ticker?pair=XBTEUR").json()["result"]["XXBTZEUR"]
    df = pd.DataFrame(columns = ["datetime", "last_trade_closed_price", "last_trade_closed_price_lot_volume"])
    df = df.append(
        {
            "datetime"                                   : datetime.datetime.utcnow(),
            "ask_price"                                  : float(Kraken_ticker["a"][0]),
            "ask_lot_volume"                             : float(Kraken_ticker["a"][2]),
            "bid_price"                                  : float(Kraken_ticker["b"][0]),
            "bid_lot_volume"                             : float(Kraken_ticker["b"][2]),
            "last_trade_closed_price"                    : float(Kraken_ticker["l"][0]),
            "last_trade_closed_price_lot_volume"         : float(Kraken_ticker["l"][1]),
            "volume_today"                               : float(Kraken_ticker["v"][0]),
            "volume_last_24_hours"                       : float(Kraken_ticker["v"][1]),
            "volume_weighted_average_price_today"        : float(Kraken_ticker["p"][0]),
            "volume_weighted_average_price_last_24_hours": float(Kraken_ticker["p"][1]),
            "number_of_trades_today"                     : float(Kraken_ticker["t"][0]),
            "number_of_trades_last_24_hours"             : float(Kraken_ticker["t"][1]),
            "low_price_today"                            : float(Kraken_ticker["l"][0]),
            "low_price_last_24_hours"                    : float(Kraken_ticker["l"][1]),
            "high_price_today"                           : float(Kraken_ticker["h"][0]),
            "high_price_last_24_hours"                   : float(Kraken_ticker["h"][1]),
            "open_price"                                 : float(Kraken_ticker["o"]),
        },
        ignore_index = True
    )
    return df

def DataFrame_last_recorded_LBC_UK_buy_ads_with_arbitrage_factors(
    filepath_LBC_UK = "LBC_UK.csv",
    filepath_Kraken = "Kraken.csv"
    ):
    if not exist_filepaths(filepaths = [filepath_LBC_UK, filepath_Kraken]):
        return False
    last_price_Kraken_GBP = last_price_Kraken_recorded(filepath_Kraken = filepath_Kraken, currency_output = "GBP")
    df = pd.read_csv(filepath_LBC_UK)
    # get all ads from last timestamp
    df = df.loc[df["datetime"] == df["datetime"].values[-1]]
    df["arbitrage_factor"] = df["temp_price_gbp"] / last_price_Kraken_GBP
    return df

def append_DataFrame_to_CSV(
    df       = None,
    filepath = None
    ):
    log.info("append DataFrame to {filepath}".format(filepath = filepath))
    df.to_csv(filepath, header = not os.path.isfile(filepath), index = False, mode = "a")

def append_arbitrage_DataFrames_to_CSV(
    filepath_LBC_UK = "LBC_UK.csv",
    filepath_Kraken = "Kraken.csv"
    ):
    append_DataFrame_to_CSV(df = DataFrame_LBC_UK_buy_ads(),        filepath = filepath_LBC_UK)
    append_DataFrame_to_CSV(df = DataFrame_Kraken_ticker_BTC_EUR(), filepath = filepath_Kraken)

def loop_append_arbitrage_DataFrames_to_CSV(
    filepath_LBC_UK = "LBC_UK.csv",
    filepath_Kraken = "Kraken.csv",
    interval        = 120
    ):
    while True:
        append_DataFrame_to_CSV(df = DataFrame_LBC_UK_buy_ads(),        filepath = filepath_LBC_UK)
        append_DataFrame_to_CSV(df = DataFrame_Kraken_ticker_BTC_EUR(), filepath = filepath_Kraken)
        log.info("loop in {interval} s".format(interval = interval))
        time.sleep(interval)

def age_timedelta_last_entry_DataFrame_CSV(
    filepath = None
    ):
    if not filepath:
        return False
    df = pd.read_csv(filepath)
    return datetime.datetime.utcnow() - pd.to_datetime(df["datetime"].values[-1]).to_pydatetime()

def arbitrage_factor_required(
    volume_EUR   = 9000,
    net_gain_EUR = 100
    ):
    """
    Return the arbitrage factor required in order to make a specified net gain
    through the changing of a specified volume.

    Kraken EUR to BTC fee:       1.6 %
    Kraken BTC withdraw fee:     0.0005 BTC (~4.36 EUR 2018-03-06T1851Z)
    LBC sell fee:                1 %
    TransferWise GBP to EUR fee: 0.35 % + 0.8 GBP (~0.90 EUR 2018-03-06T1851Z)
    """
    return (net_gain_EUR + 4.36 + 0.90) / volume_EUR + 1 + 0.016 + 0.01 + 0.0035

def last_price_Kraken_recorded(
    filepath_Kraken = "Kraken.csv",
    currency_output = "EUR"
    ):
    if not exist_filepaths(filepaths = [filepath_Kraken]):
        return False
    df_Kraken             = pd.read_csv(filepath_Kraken)
    last_price_Kraken_EUR = float(df_Kraken["last_trade_closed_price"].values[-1])
    if currency_output == "EUR":
        return last_price_Kraken_EUR
    else:
        return convert_currency(currency_input = "EUR", currency_output = currency_output, amount = last_price_Kraken_EUR)

def price_recommendation(
    arbitrage_factor      = None,
    price_lower_limit_GBP = 7000,
    exclude_usernames     = [],
    filepath_LBC_UK       = "LBC_UK.csv",
    filepath_Kraken       = "Kraken.csv",
    behaviour             = "maths"
    ):
    """
    Return a price recommendation based on the specified behaviour and a price
    lower limit, if specified. Any specified exclusion usernames are removed
    from calculations.
    """
    if not exist_filepaths(filepaths = [filepath_LBC_UK, filepath_Kraken]):
        return False
    if not arbitrage_factor:
        arbitrage_factor = arbitrage_factor_required()
    if not price_lower_limit_GBP:
        price_lower_limit_GBP = 0
    if behaviour == "calcium":
        """
        From the required arbitrage factor, calculate the price required using
        the last recorded Kraken price. Then, using the last recorded LBC UK
        data, select the next price higher on LBC UK than the minimum price
        required. From this, subtract a pseudorandom number between 0 and
        0.5 GBP.
        """
        last_price_Kraken_GBP = last_price_Kraken_recorded(filepath_Kraken = filepath_Kraken, currency_output = "GBP")
        price_required_GBP    = arbitrage_factor * last_price_Kraken_GBP
        df_LBC_UK             = pd.read_csv(filepath_LBC_UK)
        # remove any specified usernames from calculations
        for username in exclude_usernames:
            df_LBC_UK = df_LBC_UK[df_LBC_UK["seller_username"] != username]
        # get all ads from last timestamp
        df = df_LBC_UK.loc[df_LBC_UK["datetime"] == df_LBC_UK["datetime"].values[-1]]
        # get the first price greater than the price derived from the required arbitrage factor
        next_highest_price_GBP = df.loc[df["temp_price_gbp"] >= price_required_GBP]["temp_price_gbp"].values[0]
        return max(next_highest_price_GBP, price_lower_limit_GBP) - round(random.uniform(0, 0.5), 2)
    elif behaviour == "maths":
        """
        From the required arbitrage factor, calculate the price required using
        the last recorded Kraken price. Then, using the last recorded LBC UK
        data, select the higher of the calculated price and the fourth lowest
        LBC UK price. From this, subtract a pseudorandom number between 0 and
        0.5 GBP.
        """
        last_price_Kraken_GBP = last_price_Kraken_recorded(filepath_Kraken = filepath_Kraken, currency_output = "GBP")
        price_required_GBP    = arbitrage_factor * last_price_Kraken_GBP
        df_LBC_UK             = pd.read_csv(filepath_LBC_UK)
        # remove any specified usernames from calculations
        for username in exclude_usernames:
            df_LBC_UK = df_LBC_UK[df_LBC_UK["seller_username"] != username]
        # get all ads from last timestamp
        df = df_LBC_UK.loc[df_LBC_UK["datetime"] == df_LBC_UK["datetime"].values[-1]]
        return max(price_required_GBP, float(df["temp_price_gbp"].values[3])) - round(random.uniform(0, 0.5), 2)
    else:
        return False

def DataFrame_display_arbitrage():
    df = DataFrame_last_recorded_LBC_UK_buy_ads_with_arbitrage_factors()
    timestamp = df["datetime"].values[-1]
    del df["datetime"]
    del df["ad_id"]
    del df["bank_name"]
    del df["online_provider"]
    del df["payment_window_minutes"]
    del df["require_feedback_score"]
    del df["require_identification"]
    del df["require_trade_volume"]
    del df["require_trusted_by_advertiser"]
    del df["sms_verification_required"]
    del df["trade_type"]
    del df["trusted_required"]
    del df["volume_coefficient_btc"]
    return df

def display_arbitrage():
    df = DataFrame_display_arbitrage()
    log.info(df["datetime"].values[-1])
    log.info(pyprel.Table(contents = pyprel.table_DataFrame(df)))

def arbitrage_ok(
    filepath_LBC_UK = "LBC_UK.csv",
    filepath_Kraken = "Kraken.csv"
    ):
    """
    Check recorded arbitrage data for currency. If the data is not current (of
    age less than 5 minutes), return False.
    """
    if not exist_filepaths(filepaths = [filepath_LBC_UK, filepath_Kraken]):
        return False
    filepaths = [filepath_LBC_UK, filepath_Kraken]
    status = {}
    for filepath in filepaths:
        status[filepath] = age_timedelta_last_entry_DataFrame_CSV(filepath = filepath)
    filepaths_old = [k for k, v in list(status.items()) if v > datetime.timedelta(minutes = 5)]
    for filepath in filepaths_old:
        log.info("{filepath} not updated recently".format(filepath = filepath))
    if filepaths_old:
        return False
    else:
        return True

def exist_filepaths(
    filepaths = None
    ):
    if not filepaths:
        log.error("no filepaths specified")
        return False
    status = {}
    for filepath in filepaths:
        status[filepath] = os.path.isfile(filepath)
    filepaths_nonexistent = [k for k, v in list(status.items()) if not v]
    for filepath in filepaths_nonexistent:
        log.info("{filepath} not found".format(filepath = filepath))
    if filepaths_nonexistent:
        return False
    else:
        return True

def convert_currency(
    currency_input  = None,
    currency_output = None,
    amount          = 1
    ):
    try:
        r = requests.get("https://finance.google.com/finance/converter?a={amount}&from={currency_input}&to={currency_output}".format(
            amount          = amount,
            currency_input  = currency_input,
            currency_output = currency_output
        ))
        soup = BeautifulSoup(r.content, "lxml")
        return float(soup.findAll("span", {"class": "bld"})[0].text.strip(currency_output).strip())
    except:
        log.warning("Google Finance inaccessible for currency conversion, switching to CurrencyConverter")
        try:
            return currency_converter.CurrencyConverter().convert(amount, currency_input, currency_output)
        except:
            return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# loop_save_Santander_to_CSV                                                   #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program gets the recent transactions of a Santander account using       #
# Firefox and Selenium and saves them to CSV.                                  #
#                                                                              #
# copyright (C) 2018 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help        display help message
    --version         display version and exit

    --CSV=FILEPATH    bank account CSV                       [default: ./Santander.csv]

    --interval=INT    time between recordings (s)            [default: 30]
    --sleep_time=INT  time between website interactions (s)  [default: 6]
"""

import docopt
import os

import denarius.Santander

name    = "loop_save_Santander_to_CSV"
version = "2018-01-21T0227Z"

def main(options):

    filepath_CSV = os.path.expanduser(options["--CSV"])
    interval     =                int(options["--interval"])
    sleep_time   =                int(options["--sleep_time"])

    denarius.Santander.loop_save_transactions_to_CSV(
        filepath_CSV = filepath_CSV,
        sleep_time   = sleep_time,
        interval     = interval
    )

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# denarius_save_stock_prices                                                   #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program saves Google Finance stock prices to CSV.                       #
#                                                                              #
# copyright (C) 2017 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help               display help message
    --version                display version and exit

    --instruments=TEXT       comma-separated instruments list to save (none for
                             default list)          [default: none]
    --filename_CSV=FILENAME  filename of output CSV [default: stocks.csv]
    --filename_PKL=FILENAME  filename of output PKL [default: stocks.pkl]
    --output_CSV=BOOL        save to CSV            [default: true]
    --output_PKL=BOOL        save to PKL            [default: true]
"""

import docopt
import io
try:
    from urllib.request import urlopen
except:
    from urllib2 import urlopen

import denarius
import pandas as pd

name    = "denarius_save_stock_prices"
version = "2017-10-08T1836Z"
logo    = None

def main(options):

    instruments  = options["--instruments"]
    filename_CSV = options["--filename_CSV"]
    filename_PKL = options["--filename_PKL"]
    output_CSV   = options["--output_CSV"].lower() == "true"
    output_PKL   = options["--output_PKL"].lower() == "true"

    if instruments == "none":
        instruments = [
            "AAPL",
            "GOOGL",
            "GOOG",
            "MSFT",
            #"FB",
            "ORCL",
            #"TSM",
            "INTC",
            "CSCO",
            #"IBM",
            #"SAP",
            #"AVGO",
            #"DCM",
            "NVDA",
            "QCOM"
        ]
    else:
        instruments = instruments.split(",")

    dfs = []
    for instrument in instruments:
        dfs.append(denarius.instrument_DataFrame(instrument = instrument))
    df = denarius.merge_instrument_DataFrames(dfs = dfs)

    if output_CSV:
        print("save to " + filename_CSV)
        df.to_csv(filename_CSV)
    if output_PKL:
        print("save to " + filename_PKL)
        df.to_pickle(filename_PKL)

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# denarius_loop_save_Nanopool                                                  #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program loop records Nanopool data for an address.                      #
#                                                                              #
# copyright (C) 2018 Will Breaden Madden                                       #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help        display help message
    --version         display version and exit

    --addresses=TEXT  address or comma-separated addresses
    --CSV=FILENAME    filename for saving CSV data                     [default: Nanopool.csv]
    --interval=FLOAT  loop time interval (s)                           [default: 120]
    --print_table     print table of current data each recording loop
"""

import datetime
import docopt
import os
import requests
import sys
import time

import pandas as pd
import pyprel

name    = "denarius_loop_save_Nanopool"
version = "2018-02-03T1122Z"

def main(options):

    addresses    =                    options["--addresses"]
    filepath_CSV = os.path.expanduser(options["--CSV"])
    interval     =              float(options["--interval"])
    print_table  =                    options["--print_table"]
    if not addresses:
        print("error -- no address specified")
        sys.exit()
    addresses = addresses.split(",")
    while True:
        for address in addresses:
            try:
                response_user     = requests.get("https://api.nanopool.org/v1/eth/user/" + address).json()
                time.sleep(3)
                response_earnings = requests.get("https://api.nanopool.org/v1/eth/approximated_earnings/" + response_user["data"]["hashrate"]).json()
                time.sleep(3)
                hashrate_pool     = requests.get("https://api.nanopool.org/v1/eth/pool/hashrate").json()["data"]
                time.sleep(3)
                pool_miners       = requests.get("https://api.nanopool.org/v1/eth/pool/activeminers").json()["data"]
                time.sleep(3)
                pool_workers      = requests.get("https://api.nanopool.org/v1/eth/pool/activeworkers").json()["data"]
                df = pd.DataFrame(columns = ["datetime"])
                df = df.append(
                    {
                        "datetime"               : datetime.datetime.utcnow(),
                        "account"                : response_user["data"]["account"    ],
                        "balance"                : response_user["data"]["balance"    ],
                        "hashrate"               : response_user["data"]["hashrate"   ],
                        "hashrate1hr"            : response_user["data"]["avgHashrate"]["h1"],
                        "hashrate3hr"            : response_user["data"]["avgHashrate"]["h3"],
                        "hashrate6hr"            : response_user["data"]["avgHashrate"]["h6"],
                        "hashrate12hr"           : response_user["data"]["avgHashrate"]["h12"],
                        "hashrate24hr"           : response_user["data"]["avgHashrate"]["h24"],
                        "earnings_per_minute_ETH": response_earnings["data"]["minute"]["coins"   ],
                        "earnings_per_minute_BTC": response_earnings["data"]["minute"]["bitcoins"],
                        "earnings_per_minute_EUR": response_earnings["data"]["minute"]["euros"   ],
                        "earnings_per_hour_ETH"  : response_earnings["data"]["hour"  ]["coins"   ],
                        "earnings_per_hour_BTC"  : response_earnings["data"]["hour"  ]["bitcoins"],
                        "earnings_per_hour_EUR"  : response_earnings["data"]["hour"  ]["euros"   ],
                        "earnings_per_day_ETH"   : response_earnings["data"]["day"   ]["coins"   ],
                        "earnings_per_day_BTC"   : response_earnings["data"]["day"   ]["bitcoins"],
                        "earnings_per_day_EUR"   : response_earnings["data"]["day"   ]["euros"   ],
                        "earnings_per_week_ETH"  : response_earnings["data"]["week"  ]["coins"   ],
                        "earnings_per_week_BTC"  : response_earnings["data"]["week"  ]["bitcoins"],
                        "earnings_per_week_EUR"  : response_earnings["data"]["week"  ]["euros"   ],
                        "earnings_per_month_ETH" : response_earnings["data"]["month" ]["coins"   ],
                        "earnings_per_month_BTC" : response_earnings["data"]["month" ]["bitcoins"],
                        "earnings_per_month_EUR" : response_earnings["data"]["month" ]["euros"   ],
                        "hashrate_pool"          : hashrate_pool,
                        "pool_miners"            : pool_miners,
                        "pool_workers"           : pool_workers
                    },
                    ignore_index = True
                )
                df.to_csv(filepath_CSV, header = not os.path.isfile(filepath_CSV), index = False, mode = "a")
                if print_table:
                    print(pyprel.Table(contents = pyprel.table_DataFrame(df = df)))
            except:
                print("error")
                pass
            time.sleep(30)
        print("\nwait {interval} s".format(interval = interval))
        time.sleep(interval)

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)
